@@ -7,6 +7,7 @@ with 'Document::OOXML::Part';
 
 # ABSTRACT: WordprocessingML document part handling
 
+use Carp;
 use List::Util qw(first);
 use XML::LibXML;
 
@@ -164,7 +165,11 @@ sub style_text {
         my ($run_props) = $self->xpc->findnodes('ancestor::w:r[1]/w:rPr', $text_node);
         if (!$run_props) {
             my ($run) = $self->xpc->findnodes('ancestor::w:r[1]', $text_node);
-            $run_props = $run->addNewChild($run->namespaceURI => 'rPr');
+
+            my $run_ns = $run->namespaceURI;
+            $run_props = $self->xml->createElement('rPr');
+            $run_props->setNamespace($run_ns => $run->lookupNamespacePrefix($run_ns));
+            $run->insertBefore($run_props, $run->firstChild);
         }
 
         if (exists $options{bold}) {
@@ -190,21 +195,59 @@ sub style_text {
         }
 
         if (exists $options{color}) {
-            my ($color) = $self->xpc->findnodes('./w:color', $run_props);
+            my $color_to_set = _check_color_syntax($options{color});
 
-            if (!$color && $options{color}) {
-                $color = $run_props->addNewChild($run_props->namespaceURI => 'color');
-            } 
+            my ($color_element) = $self->xpc->findnodes('./w:color', $run_props);
 
-            if ($color && $options{color}) {
-                $color->setAttributeNS(
-                    $color->namespaceURI,
-                    'val' => $options{color}
+            if (!$color_element && $options{color}) {
+                $color_element = $run_props->addNewChild($run_props->namespaceURI => 'color');
+            }
+
+            if ($color_element && $options{color}) {
+                $color_element->setAttributeNS(
+                    $color_element->namespaceURI,
+                    'val' => $color_to_set
                 );
             }
 
-            if ($color && !$options{color}) {
-                $run_props->removeChild($color);
+            if ($color_element && not defined $options{color}) {
+                $run_props->removeChild($color_element);
+            }
+        }
+
+        if (exists $options{underline_style}) {
+            my $underline_style = _check_underline_style($options{underline_style});
+
+            my ($underline_element) = $self->xpc->findnodes('./w:u', $run_props);
+
+            if (!$underline_element && $options{underline_style}) {
+                $underline_element = $run_props->addNewChild($run_props->namespaceURI => 'u');
+            }
+
+            if ($underline_element && $options{underline_style}) {
+                $underline_element->setAttributeNS(
+                    $underline_element->namespaceURI,
+                    'val' => $options{underline_style}
+                );
+
+                if (exists $options{underline_color}) {
+                    if (defined $options{underline_color}) {
+                        my $color_to_set = _check_color_syntax($options{underline_color});
+
+                        $underline_element->setAttributeNS(
+                            $underline_element->namespaceURI,
+                            'color' => $color_to_set,
+                        );
+                    } else {
+                        $underline_element->removeAttributeNS(
+                            $underline_element->namespaceURI => 'color'
+                        );
+                    }
+                }
+            }
+
+            if ($underline_element && not defined $options{underline_style}) {
+                $run_props->removeChild($underline_element);
             }
         }
     }
@@ -212,6 +255,49 @@ sub style_text {
     return;
 }
 
+sub _check_color_syntax {
+    my $color = shift;
+
+    return        if not defined $color;
+    return $color if $color eq 'auto';
+    return uc($color) if $color =~ /^[0-9a-fA-F]{6}$/;
+
+    croak("Invalid color value: '$color'");
+}
+
+my %underline_styles = (
+    map { $_ => 1 } qw(
+        dash
+        dashDotDotHeavy
+        dashDotHeavy
+        dashedHeavy
+        dashLong
+        dashLongHeavy
+        dotDash
+        dotDotDash
+        dotted
+        dottedHeavy
+        double
+        none
+        single
+        thick
+        wave
+        wavyDouble
+        wavyHeavy
+        words
+    )
+);
+
+sub _check_underline_style {
+    my $underline_style = shift;
+
+
+    return if not defined $underline_style;
+    return $underline_style if exists $underline_styles{$underline_style};
+
+    croak("Invalid underline style: '$underline_style'");
+}
+
 
 sub remove_spellcheck_markers {
     my $self = shift;
@@ -438,7 +524,7 @@ Document::OOXML::Part::WordprocessingML - WordprocessingML document part handlin
 
 =head1 VERSION
 
-version 0.180741
+version 0.180742
 
 =head1 ATTRIBUTES
 
@@ -500,6 +586,37 @@ false, matching text will be un-bolded.
 Boolean value. If true, matching text will be made I<italic>. If specified and
 false, matching text will be un-italiced.
 
+=item * underline_style
+
+String value. Matching text will get an underline of the specified style. If
+the key exists specified and the value is c<undef>, underline will be removed
+from the matching text.
+
+Allowed underline styles are:
+
+    dash
+    dashDotDotHeavy
+    dashDotHeavy
+    dashedHeavy
+    dashLong
+    dashLongHeavy
+    dotDash
+    dotDotDash
+    dotted
+    dottedHeavy
+    double
+    none
+    single
+    thick
+    wave
+    wavyDouble
+    wavyHeavy
+    words
+
+=item * underline_color
+
+Color value that will be used for the underline only.
+
 =item * color
 
 Color value, matching text will be given this color.
